<?php

namespace Drupal\k8s\Service;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Component\Serialization\Yaml;

use Drupal\k8s\Entity\K8sNode;
use Drupal\k8s\Entity\K8sNamespace;
use Drupal\k8s\Entity\K8sPod;
use Drupal\k8s\Entity\K8sDeployment;
use Drupal\k8s\Entity\K8sServiceEntity;
use Drupal\k8s\Entity\K8sCronJob;
use Drupal\k8s\Entity\K8sJob;
use Drupal\k8s\Entity\K8sResourceQuota;

/**
 * Entity update methods for Batch API processing.
 */
class K8sBatchOperations {

  /**
   * The finish callback function.
   *
   * Deletes stale entities from the database.
   *
   * @param string $entity_type
   *   The entity type.
   * @param array $stale
   *   The stale entities to delete.
   * @param bool $clear
   *   TRUE to clear entities, FALSE keep them.
   */
  public static function finished($entity_type, array $stale, $clear = TRUE) {
    $entity_type_manager = \Drupal::entityTypeManager();
    if (count($stale) && $clear == TRUE) {
      $entity_type_manager->getStorage($entity_type)->delete($stale);
    }
  }

  /**
   * Update or create a k8s node entity.
   *
   * @param string $cloud_context
   *   The cloud context.
   * @param array $node
   *   The node array.
   */
  public static function updateNode($cloud_context, array $node) {
    $k8s_service = \Drupal::service('k8s');
    $k8s_service->setCloudContext($cloud_context);

    $timestamp = time();
    $name = $node['metadata']['name'];
    $entity_id = $k8s_service->getEntityId('k8s_node', 'name', $name);

    $status = '';
    $last_condition = end($node['status']['conditions']);
    if (!empty($last_condition)) {
      $status = $last_condition['type'];
    }

    if (!empty($entity_id)) {
      $entity = K8sNode::load($entity_id);
    }
    else {
      $entity = K8sNode::create([
        'cloud_context' => $cloud_context,
        'name' => $name,
        'created' => strtotime($node['metadata']['creationTimestamp']),
        'changed' => strtotime($node['metadata']['creationTimestamp']),
        'refreshed' => $timestamp,
      ]);
    }

    $entity->setStatus($status);

    // Labels.
    self::setKeyValueTypeFieldValue($entity, 'labels', $node['metadata']['labels']);

    // Annotations.
    self::setKeyValueTypeFieldValue($entity, 'annotations', $node['metadata']['annotations']);

    // Addresses.
    $map = [];
    foreach ($node['status']['addresses'] as $address) {
      $map[$address['type']] = $address['address'];
    }
    self::setKeyValueTypeFieldValue($entity, 'addresses', $map);

    $entity->setPodCidr($node['spec']['podCIDR']);
    $entity->setProviderId($node['spec']['providerID']);
    $entity->setUnschedulable(isset($node['spec']['unschedulable']) ?: FALSE);
    $entity->setMachineId($node['status']['nodeInfo']['machineID']);
    $entity->setSystemUuid($node['status']['nodeInfo']['systemUUID']);
    $entity->setBootId($node['status']['nodeInfo']['bootID']);
    $entity->setKernelVersion($node['status']['nodeInfo']['kernelVersion']);
    $entity->setOsImage($node['status']['nodeInfo']['osImage']);
    $entity->setContainerRuntimeVersion($node['status']['nodeInfo']['containerRuntimeVersion']);
    $entity->setKubeletVersion($node['status']['nodeInfo']['kubeletVersion']);
    $entity->setKubeProxyVersion($node['status']['nodeInfo']['kubeProxyVersion']);
    $entity->setOperatingSystem($node['status']['nodeInfo']['operatingSystem']);
    $entity->setArchitecture($node['status']['nodeInfo']['architecture']);

    $entity->setRefreshed($timestamp);
    $entity->save();
  }

  /**
   * Update or create a k8s namespace entity.
   *
   * @param string $cloud_context
   *   The cloud context.
   * @param array $namespace
   *   The namespace array.
   */
  public static function updateNamespace($cloud_context, array $namespace) {
    $k8s_service = \Drupal::service('k8s');
    $k8s_service->setCloudContext($cloud_context);

    $timestamp = time();
    $name = $namespace['metadata']['name'];
    $entity_id = $k8s_service->getEntityId('k8s_namespace', 'name', $name);

    $status = $namespace['status']['phase'];

    if (!empty($entity_id)) {
      $entity = K8sNamespace::load($entity_id);
    }
    else {
      $entity = K8sNamespace::create([
        'cloud_context' => $cloud_context,
        'name' => $name,
        'created' => strtotime($namespace['metadata']['creationTimestamp']),
        'changed' => strtotime($namespace['metadata']['creationTimestamp']),
        'refreshed' => $timestamp,
      ]);
    }

    $entity->setStatus($status);

    // Labels.
    $labels = [];
    if (!empty($namespace['metadata']['labels'])) {
      $labels = $namespace['metadata']['labels'];
    }
    self::setKeyValueTypeFieldValue($entity, 'labels', $labels);

    $entity->setRefreshed($timestamp);
    $entity->save();
  }

  /**
   * Update or create a k8s pod entity.
   *
   * @param string $cloud_context
   *   The cloud context.
   * @param array $pod
   *   The pod array.
   */
  public static function updatePod($cloud_context, array $pod) {
    $k8s_service = \Drupal::service('k8s');
    $k8s_service->setCloudContext($cloud_context);

    $timestamp = time();
    $name = $pod['metadata']['name'];
    $namespace = $pod['metadata']['namespace'];
    $entity_id = $k8s_service->getEntityId(
      'k8s_pod',
      'name',
      $name,
      ['namespace' => $namespace]
    );

    if (!empty($entity_id)) {
      $entity = K8sPod::load($entity_id);
    }
    else {
      $entity = K8sPod::create([
        'cloud_context' => $cloud_context,
        'name' => $name,
        'created' => strtotime($pod['metadata']['creationTimestamp']),
        'changed' => strtotime($pod['metadata']['creationTimestamp']),
        'refreshed' => $timestamp,
      ]);
    }

    // Labels.
    self::setKeyValueTypeFieldValue(
      $entity,
      'labels',
      isset($pod['metadata']['labels']) ? $pod['metadata']['labels'] : []
    );

    // Annotations.
    self::setKeyValueTypeFieldValue(
      $entity,
      'annotations',
      isset($pod['metadata']['annotations']) ? $pod['metadata']['annotations'] : []
    );

    // Containers.
    $containers = [];
    foreach ($pod['spec']['containers'] as $container_data) {
      $containers[] = Yaml::encode($container_data);
    }
    $entity->setContainers($containers);

    // Restarts.
    if (!empty($pod['status']['containerStatuses'])) {
      $entity->setRestarts($pod['status']['containerStatuses'][0]['restartCount']);
    }

    // Detail.
    $entity->setDetail(Yaml::encode($pod));

    $entity->setNamespace($pod['metadata']['namespace']);
    $entity->setStatus($pod['status']['phase']);
    $entity->setQosClass($pod['status']['qosClass']);
    $entity->setNodeName($pod['spec']['nodeName']);
    $entity->setPodIp($pod['status']['podIP']);

    $entity->setRefreshed($timestamp);
    $entity->save();
  }

  /**
   * Update or create a k8s deployment entity.
   *
   * @param string $cloud_context
   *   The cloud context.
   * @param array $deployment
   *   The deployment array.
   */
  public static function updateDeployment($cloud_context, array $deployment) {
    $k8s_service = \Drupal::service('k8s');
    $k8s_service->setCloudContext($cloud_context);

    $timestamp = time();
    $name = $deployment['metadata']['name'];
    $namespace = $deployment['metadata']['namespace'];
    $entity_id = $k8s_service->getEntityId(
      'k8s_deployment',
      'name',
      $name,
      ['namespace' => $namespace]
    );

    if (!empty($entity_id)) {
      $entity = K8sDeployment::load($entity_id);
    }
    else {
      $entity = K8sDeployment::create([
        'cloud_context' => $cloud_context,
        'name' => $name,
        'created' => strtotime($deployment['metadata']['creationTimestamp']),
        'changed' => strtotime($deployment['metadata']['creationTimestamp']),
        'refreshed' => $timestamp,
      ]);
    }

    // Labels.
    self::setKeyValueTypeFieldValue(
      $entity,
      'labels',
      isset($deployment['metadata']['labels']) ? $deployment['metadata']['labels'] : []
    );

    // Annotations.
    self::setKeyValueTypeFieldValue(
      $entity,
      'annotations',
      isset($deployment['metadata']['annotations']) ? $deployment['metadata']['annotations'] : []
    );

    // Detail.
    $entity->setDetail(Yaml::encode($deployment));

    $entity->setNamespace($deployment['metadata']['namespace']);
    $entity->setStrategy($deployment['spec']['strategy']['type']);
    $entity->setMinReadySeconds(isset($deployment['spec']['minReadySeconds']) ? $deployment['spec']['minReadySeconds'] : 0);
    $entity->setRevisionHistoryLimit($deployment['spec']['revisionHistoryLimit']);
    $entity->setAvailableReplicas(isset($deployment['status']['availableReplicas']) ? $deployment['status']['availableReplicas'] : 0);
    $entity->setCollisionCount(isset($deployment['status']['collisionCount']) ? $deployment['status']['collisionCount'] : 0);
    $entity->setObservedGeneration(isset($deployment['status']['observedGeneration']) ? $deployment['status']['observedGeneration'] : 0);
    $entity->setReadyReplicas(isset($deployment['status']['readyReplicas']) ? $deployment['status']['readyReplicas'] : 0);
    $entity->setReplicas(isset($deployment['status']['replicas']) ? $deployment['status']['replicas'] : 0);
    $entity->setUnavailableReplicas(isset($deployment['status']['unavailableReplicas']) ? $deployment['status']['unavailableReplicas'] : 0);
    $entity->setUpdatedReplicas(isset($deployment['status']['updatedReplicas']) ? $deployment['status']['updatedReplicas'] : 0);

    $entity->setRefreshed($timestamp);
    $entity->save();
  }

  /**
   * Update or create a k8s service entity.
   *
   * @param string $cloud_context
   *   The cloud context.
   * @param array $service
   *   The service array.
   */
  public static function updateService($cloud_context, array $service) {
    $k8s_service = \Drupal::service('k8s');
    $k8s_service->setCloudContext($cloud_context);

    $timestamp = time();
    $name = $service['metadata']['name'];
    $namespace = $service['metadata']['namespace'];
    $entity_id = $k8s_service->getEntityId(
      'k8s_service',
      'name',
      $name,
      ['namespace' => $namespace]
    );

    if (!empty($entity_id)) {
      $entity = K8sServiceEntity::load($entity_id);
    }
    else {
      $entity = K8sServiceEntity::create([
        'cloud_context' => $cloud_context,
        'name' => $name,
        'created' => strtotime($service['metadata']['creationTimestamp']),
        'changed' => strtotime($service['metadata']['creationTimestamp']),
        'refreshed' => $timestamp,
      ]);
    }

    // Labels.
    self::setKeyValueTypeFieldValue(
      $entity,
      'labels',
      isset($service['metadata']['labels']) ? $service['metadata']['labels'] : []
    );

    // Annotations.
    self::setKeyValueTypeFieldValue(
      $entity,
      'annotations',
      isset($service['metadata']['annotations']) ? $service['metadata']['annotations'] : []
    );

    // Selector.
    self::setKeyValueTypeFieldValue(
      $entity,
      'selector',
      isset($service['spec']['selector']) ? $service['spec']['selector'] : []
    );

    $namespace = $service['metadata']['namespace'];

    // Internal endpoints.
    $internal_endpoints = [];
    foreach ($service['spec']['ports'] as $port) {
      $internal_endpoints[] = sprintf(
        '%s.%s:%s %s',
        $name,
        $namespace,
        $port['port'],
        $port['protocol']
      );

      if (isset($port['nodePort'])) {
        $internal_endpoints[] = sprintf(
          '%s.%s:%s %s',
          $name,
          $namespace,
          $port['nodePort'],
          $port['protocol']
        );
      }
    }
    $entity->setInternalEndpoints($internal_endpoints);

    // External endpoints.
    if ($service['spec']['type'] == 'LoadBalancer') {
      $external_endpoints = [];

      if (isset($service['status']['loadBalancer']['ingress'])) {
        foreach ($service['status']['loadBalancer']['ingress'] as $lb) {
          foreach ($service['spec']['ports'] as $port) {
            $external_endpoints[] = sprintf(
              '%s:%s',
              $lb['hostname'],
              $port['port']
            );
          }
        }
      }

      $entity->setExternalEndpoints($external_endpoints);
    }

    // Detail.
    $entity->setDetail(Yaml::encode($service));

    $entity->setNamespace($namespace);
    $entity->setType($service['spec']['type']);
    $entity->setSessionAffinity($service['spec']['sessionAffinity']);
    $entity->setClusterIp($service['spec']['clusterIP']);

    $entity->setRefreshed($timestamp);
    $entity->save();
  }

  /**
   * Update or create a k8s cron job entity.
   *
   * @param string $cloud_context
   *   The cloud context.
   * @param array $cron_job
   *   The cron job array.
   */
  public static function updateCronJob($cloud_context, array $cron_job) {
    $k8s_service = \Drupal::service('k8s');
    $k8s_service->setCloudContext($cloud_context);

    $timestamp = time();
    $name = $cron_job['metadata']['name'];
    $namespace = $cron_job['metadata']['namespace'];
    $entity_id = $k8s_service->getEntityId(
      'k8s_cron_job',
      'name',
      $name,
      ['namespace' => $namespace]
    );

    if (!empty($entity_id)) {
      $entity = K8sCronJob::load($entity_id);
    }
    else {
      $entity = K8sCronJob::create([
        'cloud_context' => $cloud_context,
        'name' => $name,
        'created' => strtotime($cron_job['metadata']['creationTimestamp']),
        'changed' => strtotime($cron_job['metadata']['creationTimestamp']),
        'refreshed' => $timestamp,
      ]);
    }

    // Labels.
    self::setKeyValueTypeFieldValue(
      $entity,
      'labels',
      isset($cron_job['metadata']['labels']) ? $cron_job['metadata']['labels'] : []
    );

    // Annotations.
    self::setKeyValueTypeFieldValue(
      $entity,
      'annotations',
      isset($cron_job['metadata']['annotations']) ? $cron_job['metadata']['annotations'] : []
    );

    $namespace = $cron_job['metadata']['namespace'];

    // Active.
    if (isset($cron_job['status']['active'])) {
      $entity->setActive(count($cron_job['status']['active']));
    }
    else {
      $entity->setActive(0);
    }

    // Detail.
    $entity->setDetail(Yaml::encode($cron_job));

    $entity->setNamespace($namespace);
    $entity->setSchedule($cron_job['spec']['schedule']);
    $entity->setSuspend($cron_job['spec']['suspend']);

    if (!empty($cron_job['status']['lastScheduleTime'])) {
      $entity->setLastScheduleTime(strtotime($cron_job['status']['lastScheduleTime']));
    }
    $entity->setConcurrencyPolicy($cron_job['spec']['concurrencyPolicy']);
    if (isset($cron_job['spec']['startingDeadlineSeconds'])) {
      $entity->setStartingDeadlineSeconds($cron_job['spec']['startingDeadlineSeconds']);
    }

    $entity->setRefreshed($timestamp);
    $entity->save();
  }

  /**
   * Update or create a k8s job entity.
   *
   * @param string $cloud_context
   *   The cloud context.
   * @param array $job
   *   The job array.
   */
  public static function updateJob($cloud_context, array $job) {
    $k8s_service = \Drupal::service('k8s');
    $k8s_service->setCloudContext($cloud_context);

    $timestamp = time();
    $name = $job['metadata']['name'];
    $namespace = $job['metadata']['namespace'];
    $entity_id = $k8s_service->getEntityId(
      'k8s_job',
      'name',
      $name,
      ['namespace' => $namespace]
    );

    if (!empty($entity_id)) {
      $entity = K8sJob::load($entity_id);
    }
    else {
      $entity = K8sJob::create([
        'cloud_context' => $cloud_context,
        'name' => $name,
        'created' => strtotime($job['metadata']['creationTimestamp']),
        'changed' => strtotime($job['metadata']['creationTimestamp']),
        'refreshed' => $timestamp,
      ]);
    }

    // Labels.
    self::setKeyValueTypeFieldValue(
      $entity,
      'labels',
      isset($job['metadata']['labels']) ? $job['metadata']['labels'] : []
    );

    // Annotations.
    self::setKeyValueTypeFieldValue(
      $entity,
      'annotations',
      isset($job['metadata']['annotations']) ? $job['metadata']['annotations'] : []
    );

    $namespace = $job['metadata']['namespace'];

    // Image.
    if (!empty($job['spec']['template']['spec']['containers'])) {
      $entity->setImage($job['spec']['template']['spec']['containers'][0]['image']);
    }

    // Detail.
    $entity->setDetail(Yaml::encode($job));

    $entity->setNamespace($namespace);
    $entity->setCompletions($job['spec']['completions']);
    $entity->setParallelism($job['spec']['parallelism']);

    // Active.
    if (isset($job['status']['active'])) {
      $entity->setActive($job['status']['active']);
    }
    else {
      $entity->setActive(0);
    }

    // Succeeded.
    if (isset($job['status']['succeeded'])) {
      $entity->setSucceeded($job['status']['succeeded']);
    }
    else {
      $entity->setSucceeded(0);
    }

    // Failed.
    if (isset($job['status']['failed'])) {
      $entity->setFailed($job['status']['failed']);
    }
    else {
      $entity->setFailed(0);
    }

    $entity->setRefreshed($timestamp);
    $entity->save();
  }

  /**
   * Update or create a k8s resource quota entity.
   *
   * @param string $cloud_context
   *   The cloud context.
   * @param array $resource_quota
   *   The resource quota array.
   */
  public static function updateResourceQuota($cloud_context, array $resource_quota) {
    $k8s_service = \Drupal::service('k8s');
    $k8s_service->setCloudContext($cloud_context);

    $timestamp = time();
    $name = $resource_quota['metadata']['name'];
    $namespace = $resource_quota['metadata']['namespace'];
    $entity_id = $k8s_service->getEntityId(
      'k8s_resource_quota',
      'name',
      $name,
      ['namespace' => $namespace]
    );

    if (!empty($entity_id)) {
      $entity = K8sResourceQuota::load($entity_id);
    }
    else {
      $entity = K8sResourceQuota::create([
        'cloud_context' => $cloud_context,
        'name' => $name,
        'created' => strtotime($resource_quota['metadata']['creationTimestamp']),
        'changed' => strtotime($resource_quota['metadata']['creationTimestamp']),
        'refreshed' => $timestamp,
      ]);
    }

    // Labels.
    self::setKeyValueTypeFieldValue(
      $entity,
      'labels',
      isset($resource_quota['metadata']['labels']) ? $resource_quota['metadata']['labels'] : []
    );

    // Annotations.
    self::setKeyValueTypeFieldValue(
      $entity,
      'annotations',
      isset($resource_quota['metadata']['annotations']) ? $resource_quota['metadata']['annotations'] : []
    );

    // Status hard.
    self::setKeyValueTypeFieldValue(
      $entity,
      'status_hard',
      isset($resource_quota['status']['hard']) ? $resource_quota['status']['hard'] : []
    );

    // Status used.
    self::setKeyValueTypeFieldValue(
      $entity,
      'status_used',
      isset($resource_quota['status']['used']) ? $resource_quota['status']['used'] : []
    );

    // Detail.
    $entity->setDetail(Yaml::encode($resource_quota));

    $namespace = $resource_quota['metadata']['namespace'];
    $entity->setNamespace($namespace);
    $entity->setRefreshed($timestamp);
    $entity->save();
  }

  /**
   * Set key_value type field value.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity object.
   * @param string $field_name
   *   The field name.
   * @param array $value_map
   *   The value of map type.
   */
  private static function setKeyValueTypeFieldValue(EntityInterface $entity, $field_name, array $value_map) {
    $key_values = [];
    if (!isset($value_map)) {
      $value_map = [];
    }
    foreach ($value_map as $key => $value) {
      $key_values[] = ['item_key' => $key, 'item_value' => $value];
    }

    usort($key_values, function ($a, $b) {
      return strcmp($a['item_key'], $b['item_key']);
    });

    $entity->set($field_name, $key_values);
  }

}
